@@ -2,8 +2,8 @@
 /*
  * gsf-input-iochannel.c: GIOChannel based input
  *
- * Copyright (C) 2003-2004 Rodrigo Moya (rodrigo@gnome-db.org)
- * Copyright (C) 2003-2004 Dom Lachowicz (cinamod@hotmail.com)
+ * Copyright (C) 2003-2006 Rodrigo Moya (rodrigo@gnome-db.org)
+ * Copyright (C) 2003-2006 Dom Lachowicz (cinamod@hotmail.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -16,12 +16,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
 #include <gsf-config.h>
 #include <gsf-input-iochannel.h>
+#include <gsf-input-memory.h>
 
 /**
  * gsf_input_memory_new_from_iochannel :
@@ -30,7 +31,7 @@
  *
  * Returns a new #GsfInputMemory or NULL.
  */
-GsfInputMemory * 
+GsfInput * 
 gsf_input_memory_new_from_iochannel (GIOChannel *channel,
 				     GError **err)
 {
