@@ -2,7 +2,7 @@
 /*
  * gsf-outfile-msole.c: 
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Outc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Outc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -98,10 +98,8 @@ gsf_outfile_msole_finalize (GObject *obj)
 		break;
 
 	case MSOLE_SMALL_BLOCK:
-		if (ole->content.small_block.buf != NULL) {
-			g_free (ole->content.small_block.buf);
-			ole->content.small_block.buf = NULL;
-		}
+		g_free (ole->content.small_block.buf);
+		ole->content.small_block.buf = NULL;
 		break;
 
 	case MSOLE_BIG_BLOCK:
@@ -177,7 +175,8 @@ ole_cur_block (GsfOutfileMSOle const *ole)
 static inline unsigned
 ole_bytes_left_in_block (GsfOutfileMSOle *ole)
 {
-	unsigned r = (gsf_output_tell (ole->sink) - OLE_HEADER_SIZE) % ole->bb.size;
+	/* blocks are multiples of bb.size (the header is padded out to bb.size) */
+	unsigned r = gsf_output_tell (ole->sink) % ole->bb.size;
 	return (r != 0) ? (ole->bb.size - r) : 0;
 }
 
@@ -198,9 +197,7 @@ ole_write_bat (GsfOutput *sink, guint32 block, unsigned blocks)
 {
 	guint8 buf [BAT_INDEX_SIZE];
 
-#ifdef __GNUC__
-#warning optimize this to dump a buffer in 1 step
-#endif
+/* FIXME FIXME FIXME  optimize this to dump a buffer in 1 step */
 	while (blocks-- > 1) {
 		block++;
 		GSF_LE_SET_GUINT32 (buf, block);
@@ -538,6 +535,9 @@ gsf_outfile_msole_write (GsfOutput *output,
 	return TRUE;
 }
 
+static gsf_off_t gsf_outfile_msole_vprintf (GsfOutput *output,
+	char const *format, va_list args) G_GNUC_PRINTF (2, 0);
+
 static gsf_off_t
 gsf_outfile_msole_vprintf (GsfOutput *output, char const *format, va_list args)
 {
@@ -597,8 +597,9 @@ gsf_outfile_msole_set_block_shift (GsfOutfileMSOle *ole,
 }
 
 static GsfOutput *
-gsf_outfile_msole_new_child (GsfOutfile *parent, char const *name,
-			     gboolean is_dir)
+gsf_outfile_msole_new_child (GsfOutfile *parent,
+			     char const *name, gboolean is_dir,
+			     char const *first_property_name, va_list args)
 {
 	GsfOutfileMSOle *ole_parent = (GsfOutfileMSOle *)parent;
 	GsfOutfileMSOle *child;
@@ -606,7 +607,8 @@ gsf_outfile_msole_new_child (GsfOutfile *parent, char const *name,
 	g_return_val_if_fail (ole_parent != NULL, NULL);
 	g_return_val_if_fail (ole_parent->type == MSOLE_DIR, NULL);
 
-	child = g_object_new (GSF_OUTFILE_MSOLE_TYPE, NULL);
+	child = (GsfOutfileMSOle *)g_object_new_valist (
+		GSF_OUTFILE_MSOLE_TYPE, first_property_name, args);
 	if (is_dir) {
 		child->type = MSOLE_DIR;
 		child->content.dir.children = NULL;
@@ -681,22 +683,21 @@ compute_shift (unsigned value)
 
 /**
  * gsf_outfile_msole_new_full :
- * @sink : a #GsfOutput to hold the OLE2 file
- * @bb_size : size of large blocks
- * @sb_size : size of small blocks
+ * @sink : a #GsfOutput to hold the OLE2 file.
+ * @bb_size : size of large blocks.
+ * @sb_size : size of small blocks.
  *
  * Creates the root directory of an MS OLE file and manages the addition of
  * children.
  *
- * NOTE : adds a reference to @sink
+ * <note>This adds a reference to @sink.</note>
  *
- * Returns : the new ole file handler
+ * Returns: the new ole file handler.
  **/
 GsfOutfile *
-gsf_outfile_msole_new_full (GsfOutput *sink,
-			    unsigned bb_size, unsigned sb_size)
+gsf_outfile_msole_new_full (GsfOutput *sink, guint bb_size, guint sb_size)
 {
-	static guint8 default_header [] = {
+	static guint8 const default_header [] = {
 /* 0x00 */	0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1,
 /* 0x08 */	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
 /* 0x18 */	0x3e, 0x00, 0x03, 0x00, 0xfe, 0xff, 0x09, 0x00,
@@ -750,8 +751,8 @@ gsf_outfile_msole_new_full (GsfOutput *sink,
 	gsf_output_write (sink, OLE_HEADER_SIZE, buf);
 	g_free (buf);
 
-	/* FIXME : Do we need to pad the header out to bb_size for other block
-	 * sizes ? if so do we pad with 0 or something else ? */
+	/* header must be padded out to bb.size with zeros */
+	ole_pad_zero(ole);
 
 	return GSF_OUTFILE (ole);
 }
@@ -763,9 +764,9 @@ gsf_outfile_msole_new_full (GsfOutput *sink,
  * Creates the root directory of an MS OLE file and manages the addition of
  * children.
  *
- * NOTE : adds a reference to @sink
+ * <note>This adds a reference to @sink.</note>
  *
- * Returns : the new ole file handler
+ * Returns: the new ole file handler.
  **/
 GsfOutfile *
 gsf_outfile_msole_new (GsfOutput *sink)
@@ -781,7 +782,7 @@ gsf_outfile_msole_new (GsfOutput *sink)
  *
  * Write @clsid to the directory associated with @ole.
  *
- * Returns TRUE on success.
+ * Returns: TRUE on success.
  **/
 gboolean
 gsf_outfile_msole_set_class_id (GsfOutfileMSOle *ole, guint8 const *clsid)
