@@ -2,7 +2,7 @@
 /*
  * gsf-output-gzip.c: wrapper to compress to gzipped output. See rfc1952.
  *
- * Copyright (C) 2002-2004 Jon K Hellan (hellan@acm.org)
+ * Copyright (C) 2002-2006 Jon K Hellan (hellan@acm.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -30,20 +30,18 @@
 #include <time.h>
 #include <string.h>
 
-#define Z_BUFSIZE 0x100
-
 static GObjectClass *parent_class;
 
 struct _GsfOutputGZip {
 	GsfOutput output;
 
 	GsfOutput *sink; /* compressed data */
+	gboolean raw; /* No header and no trailer.  */
 
 	z_stream  stream;
-/* 	guint8 const *gzipped_data; */
 	uLong     crc;     /* crc32 of uncompressed data */
 	size_t    isize;
-	
+
 	guint8   *buf;
 	size_t    buf_size;
 };
@@ -52,25 +50,29 @@ typedef struct {
 	GsfOutputClass output_class;
 } GsfOutputGZipClass;
 
+enum {
+	PROP_0,
+	PROP_RAW,
+	PROP_SINK
+};
+
+
 /* gzip flag byte */
 #define GZIP_ORIGINAL_NAME	0x08 /* the original is stored */
 
 static gboolean
-init_gzip (GsfOutputGZip *gzip, GError **err)
+init_gzip (GsfOutputGZip *gzip)
 {
 	int ret;
-	
+
 	ret = deflateInit2 (&gzip->stream, Z_DEFAULT_COMPRESSION,
 			    Z_DEFLATED, -MAX_WBITS, MAX_MEM_LEVEL,
 			    Z_DEFAULT_STRATEGY);
-	if (ret != Z_OK) {
-		if (err != NULL)
-			*err = g_error_new (gsf_output_error_id (), 0,
-					    "Unable to initialize deflate");
+	if (ret != Z_OK)
 		return FALSE;
-	}
+
 	if (!gzip->buf) {
-		gzip->buf_size = Z_BUFSIZE; 
+		gzip->buf_size = 0x100;
 		gzip->buf = g_new (guint8, gzip->buf_size);
 	}
 	gzip->stream.next_out  = gzip->buf;
@@ -89,7 +91,7 @@ gzip_output_header (GsfOutputGZip *gzip)
 	/* FIXME: What to do about gz extension ... ? */
 	int nlen = 0;  /* name ? strlen (name) : 0; */
 	gboolean ret;
-	
+
 	memset (buf, 0, sizeof buf);
 	memcpy (buf, gzip_signature, 3);
 	if (nlen > 0)
@@ -102,7 +104,7 @@ gzip_output_header (GsfOutputGZip *gzip)
 
 	return ret;
 }
-	
+
 /**
  * gsf_output_gzip_new :
  * @sink : The underlying data source.
@@ -115,25 +117,22 @@ gzip_output_header (GsfOutputGZip *gzip)
 GsfOutput *
 gsf_output_gzip_new (GsfOutput *sink, GError **err)
 {
-	GsfOutputGZip *gzip;
+	GsfOutput *output;
+	const GError *con_err;
 
 	g_return_val_if_fail (GSF_IS_OUTPUT (sink), NULL);
 
-	gzip = g_object_new (GSF_OUTPUT_GZIP_TYPE, NULL);
-	g_object_ref (G_OBJECT (sink));
-	gzip->sink = sink;
+	output = g_object_new (GSF_OUTPUT_GZIP_TYPE, "sink", sink, NULL);
+	con_err = gsf_output_error (output);
 
-	if (!init_gzip (gzip, err)) {
-		g_object_unref (G_OBJECT (gzip));
+	if (con_err) {
+		if (err)
+			*err = g_error_copy (con_err);
+		g_object_unref (output);
 		return NULL;
 	}
 
-	if (!gzip_output_header (gzip)) {
-		g_object_unref (G_OBJECT (gzip));
-		return NULL;
-	}
-
-	return GSF_OUTPUT (gzip);
+	return output;
 }
 
 static void
@@ -158,8 +157,10 @@ static gboolean
 gzip_output_block (GsfOutputGZip *gzip)
 {
 	size_t num_bytes = gzip->buf_size - gzip->stream.avail_out;
-	
+
 	if (!gsf_output_write (gzip->sink, num_bytes, gzip->buf)) {
+		gsf_output_set_error (GSF_OUTPUT (gzip), 0,
+				      "Failed to write");
 		return FALSE;
 	}
 	gzip->stream.next_out  = gzip->buf;
@@ -183,6 +184,8 @@ gzip_flush (GsfOutputGZip *gzip)
 		}
 	} while (zret == Z_OK);
 	if (zret != Z_STREAM_END) {
+		gsf_output_set_error (GSF_OUTPUT (gzip), 0,
+				      "Unexpected compression failure");
 		g_warning ("Unexpected error code %d from zlib during compression.",
 			   zret);
 		return FALSE;
@@ -203,7 +206,7 @@ gsf_output_gzip_write (GsfOutput *output,
 
 	gzip->stream.next_in  = (unsigned char *) data;
 	gzip->stream.avail_in = num_bytes;
-	
+
 	while (gzip->stream.avail_in > 0) {
 		int zret;
 		if (gzip->stream.avail_out == 0) {
@@ -213,6 +216,8 @@ gsf_output_gzip_write (GsfOutput *output,
 
 		zret = deflate (&gzip->stream, Z_NO_FLUSH);
 		if (zret != Z_OK) {
+			gsf_output_set_error (output, 0,
+					      "Unexpected compression failure");
 			g_warning ("Unexpected error code %d from zlib during compression.",
 				   zret);
 			return FALSE;
@@ -241,17 +246,23 @@ gsf_output_gzip_seek (G_GNUC_UNUSED GsfOutput *output,
 static gboolean
 gsf_output_gzip_close (GsfOutput *output)
 {
-	GsfOutputGZip *gzip = GSF_OUTPUT_GZIP (output);
-	guint8 buf[8];
+	if (!gsf_output_error (output)) {
+		GsfOutputGZip *gzip = GSF_OUTPUT_GZIP (output);
 
-	if (!gzip_flush (gzip))
-		return FALSE;
-	
-	/* TODO: CRC, ISIZE */
-	GSF_LE_SET_GUINT32 (buf,     gzip->crc);
-	GSF_LE_SET_GUINT32 (buf + 4, gzip->isize);
+		if (!gzip_flush (gzip))
+			return FALSE;
 
-	return gsf_output_write (gzip->sink, 8, buf);
+		if (!gzip->raw) {
+			guint8 buf[8];
+
+			GSF_LE_SET_GUINT32 (buf,     gzip->crc);
+			GSF_LE_SET_GUINT32 (buf + 4, gzip->isize);
+			if (!gsf_output_write (gzip->sink, 8, buf))
+				return FALSE;
+		}
+	}
+
+	return TRUE;
 }
 
 static void
@@ -272,19 +283,120 @@ gsf_output_gzip_init (GObject *obj)
 	gzip->buf_size		= 0;
 }
 
+static void
+gsf_output_gzip_get_property (GObject     *object,
+			      guint        property_id,
+			      GValue      *value,
+			      GParamSpec  *pspec)
+{
+	GsfOutputGZip *gzip = (GsfOutputGZip *)object;
+
+	switch (property_id) {
+	case PROP_RAW:
+		g_value_set_boolean (value, gzip->raw);
+		break;
+	case PROP_SINK:
+		g_value_set_object (value, gzip->sink);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+gsf_output_gzip_set_sink (GsfOutputGZip *gzip, GsfOutput *sink)
+{
+	if (sink)
+		g_object_ref (GSF_OUTPUT (sink));
+	if (gzip->sink)
+		g_object_unref (gzip->sink);
+	gzip->sink = sink;
+}
+
+static void
+gsf_output_gzip_set_property (GObject      *object,
+			      guint         property_id,
+			      GValue const *value,
+			      GParamSpec   *pspec)
+{
+	GsfOutputGZip *gzip = (GsfOutputGZip *)object;
+
+	switch (property_id) {
+	case PROP_RAW:
+		gzip->raw = g_value_get_boolean (value);
+		break;
+	case PROP_SINK:
+		gsf_output_gzip_set_sink (gzip, g_value_get_object (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static GObject*
+gsf_output_gzip_constructor (GType                  type,
+			     guint                  n_construct_properties,
+			     GObjectConstructParam *construct_params)
+{
+	GsfOutputGZip *gzip;
+
+	gzip = (GsfOutputGZip *)(parent_class->constructor (type,
+							    n_construct_properties,
+							    construct_params));
+
+	if (!gzip->sink)
+		gsf_output_set_error (GSF_OUTPUT (gzip),
+				      0,
+				      "NULL sink");
+	else if (!init_gzip (gzip))
+		gsf_output_set_error (GSF_OUTPUT (gzip),
+				      0,
+				      "Failed to initialize zlib structure");
+	else if (!gzip->raw && !gzip_output_header (gzip))
+		gsf_output_set_error (GSF_OUTPUT (gzip),
+				      0,
+				      "Failed to write gzip header");
+
+	return (GObject *)gzip;
+}
+
 static void
 gsf_output_gzip_class_init (GObjectClass *gobject_class)
 {
 	GsfOutputClass *output_class = GSF_OUTPUT_CLASS (gobject_class);
 
-	gobject_class->finalize = gsf_output_gzip_finalize;
-	output_class->Write	= gsf_output_gzip_write;
-	output_class->Seek	= gsf_output_gzip_seek;
-	output_class->Close	= gsf_output_gzip_close;
+	gobject_class->constructor  = gsf_output_gzip_constructor;
+	gobject_class->finalize     = gsf_output_gzip_finalize;
+	gobject_class->set_property = gsf_output_gzip_set_property;
+	gobject_class->get_property = gsf_output_gzip_get_property;
+	output_class->Write	    = gsf_output_gzip_write;
+	output_class->Seek	    = gsf_output_gzip_seek;
+	output_class->Close	    = gsf_output_gzip_close;
+
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_RAW,
+		 g_param_spec_boolean ("raw", "Raw",
+				       "Whether to write compressed data with no header/tailer.",
+				       FALSE,
+				       GSF_PARAM_STATIC |
+				       G_PARAM_READWRITE |
+				       G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_SINK,
+		 g_param_spec_object ("sink", "Sink",
+				      "Where the compressed data is written.",
+				      GSF_OUTPUT_TYPE,
+				      GSF_PARAM_STATIC |
+				      G_PARAM_READWRITE |
+				      G_PARAM_CONSTRUCT_ONLY));
 
 	parent_class = g_type_class_peek_parent (gobject_class);
 }
 
 GSF_CLASS (GsfOutputGZip, gsf_output_gzip,
-	   gsf_output_gzip_class_init, gsf_output_gzip_init, GSF_OUTPUT_TYPE)
-
+	   gsf_output_gzip_class_init, gsf_output_gzip_init,
+	   GSF_OUTPUT_TYPE)
