@@ -2,7 +2,7 @@
 /*
  * gsf-utils.c: 
  *
- * Copyright (C) 2002-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2002-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2.1 of the GNU Lesser General Public
@@ -22,6 +22,45 @@
 #include <gsf-config.h>
 #include <gsf/gsf-utils.h>
 #include <gsf/gsf-input.h>
+#include <gsf/gsf-doc-meta-data.h>
+#include <gsf/gsf-docprop-vector.h>
+#include <gsf/gsf-impl-utils.h>
+
+#include <gsf/gsf-infile.h>
+#include <gsf/gsf-infile-msole.h>
+#include <gsf/gsf-infile-msvba.h>
+#include <gsf/gsf-infile-stdio.h>
+#include <gsf/gsf-infile-zip.h>
+
+#include <gsf/gsf-input.h>
+#include <gsf/gsf-input-gzip.h>
+#include <gsf/gsf-input-http.h>
+#include <gsf/gsf-input-memory.h>
+#include <gsf/gsf-input-proxy.h>
+#include <gsf/gsf-input-stdio.h>
+#include <gsf/gsf-input-textline.h>
+
+#include <gsf/gsf-output.h>
+#include <gsf/gsf-output-bzip.h>
+#include <gsf/gsf-output-csv.h>
+#include <gsf/gsf-output-gzip.h>
+#include <gsf/gsf-output-iconv.h>
+#include <gsf/gsf-output-iochannel.h>
+#include <gsf/gsf-output-memory.h>
+#include <gsf/gsf-output-stdio.h>
+
+#include <gsf/gsf-outfile.h>
+#include <gsf/gsf-outfile-msole.h>
+#include <gsf/gsf-outfile-stdio.h>
+#include <gsf/gsf-outfile-zip.h>
+
+#include <gsf/gsf-libxml.h>
+#include <gsf/gsf-blob.h>
+#include <gsf/gsf-structured-blob.h>
+#include <gsf/gsf-shared-memory.h>
+#include <gsf/gsf-clip-data.h>
+#include <gsf/gsf-open-pkg-utils.h>
+
 #include <gobject/gvaluecollector.h>
 #include <glib/gi18n-lib.h>
 
@@ -48,11 +87,32 @@ static void base64_init (void);
 G_WIN32_DLLMAIN_FOR_DLL_NAME (static, dll_name)
 #endif
 
+#ifdef _GSF_GTYPE_THREADING_FIXED
+typedef GTypeModule      GsfDummyTypeModule;
+typedef GTypeModuleClass GsfDummyTypeModuleClass;
+static gboolean
+gsf_dummy_type_module_load (GTypeModule *module)
+{
+	gsf_init_dynamic (module);
+	return TRUE;
+}
+static void
+gsf_dummy_type_module_class_init (GTypeModuleClass *gtm_class)
+{
+	gtm_class->load = gsf_dummy_type_module_load;
+}
+static GSF_CLASS (GsfDummyTypeModule, gsf_dummy_type_module,
+		  gsf_dummy_type_module_class_init, NULL,
+		  G_TYPE_TYPE_MODULE)
+
+static GTypeModule *static_type_module = NULL;
+#endif
+
 /**
  * gsf_init :
  *
  * Initializes the GSF library
- */
+ **/
 void
 gsf_init (void)
 {
@@ -72,23 +132,102 @@ gsf_init (void)
 
 	g_type_init ();
 	base64_init ();
+
+#ifdef _GSF_GTYPE_THREADING_FIXED
+	if (NULL == static_type_module) {
+		static_type_module = g_object_new (gsf_dummy_type_module_get_type(), NULL);
+		g_type_module_use (static_type_module);
+		g_type_module_set_name (static_type_module, "libgsf-builtin");
+	}
+#else
+	gsf_init_dynamic (NULL);
+#endif
 }
 
 /**
  * gsf_shutdown:
  * 
  * De-intializes the GSF library
+ * Currently does nothing.
  **/
 void
 gsf_shutdown (void)
 {
 }
 
+#ifdef _GSF_GTYPE_THREADING_FIXED
+#define REGISTER(prefix)						\
+	do {								\
+		prefix ## _register_type (module);			\
+		types = g_slist_prepend (types,				\
+			g_type_class_ref (prefix ## _get_type()));	\
+	} while (0)
+#else
+/* Assign the value to avoid compiler warnings */
+#define REGISTER(prefix)	t = prefix ## _get_type()
+#endif
+
+/**
+ * gsf_init_dynamic :
+ * @module : #GTypeModule.
+ *
+ * Initializes the GSF library and associates it with a type module @mod.
+ **/
 void
-gsf_init_dynamic (G_GNUC_UNUSED GTypeModule *module)
+gsf_init_dynamic (GTypeModule *module)
 {
+#ifndef _GSF_GTYPE_THREADING_FIXED
+	GType t;
+	if (NULL != module) {
+		g_warning ("glib's support of dynamic types is not thread safe.\n"
+			   "Support for gsf_init_dynamic has been disabled until that is fixed");
+	}
+#endif
+	REGISTER (gsf_input);
+	REGISTER (gsf_input_gzip);
+	REGISTER (gsf_input_http);
+	REGISTER (gsf_input_memory);
+	REGISTER (gsf_input_proxy);
+	REGISTER (gsf_input_stdio);
+	REGISTER (gsf_input_textline);
+
+	REGISTER (gsf_infile);
+	REGISTER (gsf_infile_msole);
+	REGISTER (gsf_infile_msvba);
+	REGISTER (gsf_infile_stdio);
+	REGISTER (gsf_infile_zip);
+
+	REGISTER (gsf_output);
+	REGISTER (gsf_output_bzip);
+	REGISTER (gsf_output_csv_quoting_mode);
+	REGISTER (gsf_output_csv);
+	REGISTER (gsf_output_gzip);
+	REGISTER (gsf_output_iconv);
+	REGISTER (gsf_output_iochannel);
+	REGISTER (gsf_output_memory);
+	REGISTER (gsf_output_stdio);
+
+	REGISTER (gsf_outfile);
+	REGISTER (gsf_outfile_msole);
+	REGISTER (gsf_outfile_stdio);
+	REGISTER (gsf_outfile_zip);
+	REGISTER (gsf_outfile_open_pkg);
+
+	REGISTER (gsf_shared_memory);
+	REGISTER (gsf_structured_blob);
+	REGISTER (gsf_xml_out);
+	REGISTER (gsf_blob);
+	REGISTER (gsf_clip_data);
+	REGISTER (gsf_doc_meta_data);
+	REGISTER (gsf_docprop_vector);
 }
 
+/**
+ * gsf_shutdown:
+ * 
+ * De-intializes the GSF library from a type module.
+ * Currently does nothing.
+ **/
 void
 gsf_shutdown_dynamic (G_GNUC_UNUSED GTypeModule *module)
 {
@@ -97,20 +236,31 @@ gsf_shutdown_dynamic (G_GNUC_UNUSED GTypeModule *module)
 static void
 gsf_mem_dump_full (guint8 const *ptr, size_t len, gsf_off_t offset)
 {
-	gsf_off_t i, j, off;
+	static const char hexdigit[16] = "0123456789abcdef";
 
-	for (i = 0 ; i < (len+15)/16 ; i++) {
-		g_print ("%8lx | ", (long)(i*16 + offset));
-		for (j = 0;j < 16; j++) {
-			off = j + (i << 4);
-			off<len ? g_print("%02x ", ptr[off]) : g_print("XX ");
+	while (len > 0) {
+		char hexpart[3 * 16 + 1], *phex = hexpart;
+		char pic[17];
+		size_t j;
+		for (j = 0; j < 16; j++) {
+			if (len > 0) {
+				*phex++ = hexdigit[*ptr >> 4];
+				*phex++ = hexdigit[*ptr & 0xf];
+				pic[j] = (*ptr >= '!' && *ptr < 127 ? *ptr : '.');
+				len--;
+				ptr++;
+			} else {
+				*phex++ = 'X';
+				*phex++ = 'X';
+				pic[j] = '*';
+			}
+			*phex++ = ' ';
 		}
-		g_print ("| ");
-		for (j = 0 ; j < 16 ; j++) {
-			off = j + (i<<4);
-			g_print ("%c", off < len ? (ptr[off] >= '!' && ptr[off] < 127 ? ptr[off] : '.') : '*');
-		}
-		g_print ("\n");
+		hexpart[3 * 16] = 0;
+		pic[16] = 0 ;
+
+		g_print ("%8lx | %s| %s\n", (long)offset, hexpart, pic);
+		offset += 16;
 	}
 }
 
